.ac_wrapper {
    position: relative;
}

.ac_results {
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
    padding: 0px;
    border: 1px solid black;
    background-color: white;
    overflow: hidden;
    z-index: 99999;
}

.ac_results ul {
    width: 100%;
    list-style-position: outside;
    list-style: none;
    padding: 0;
    margin: 0;
}

.ac_results li {
    margin: 0px;
    padding: 2px 5px;
    cursor: default;
    display: block;
    /*
    if width will be 100% horizontal scrollbar will apear
    when scroll mode will be used
    */
    /*width: 100%;*/
    font: menu;
    font-size: 12px;
    /*
    it is very important, if line-height not setted or setted
    in relative units scroll will be broken in firefox
    */
    line-height: 16px;
    overflow: hidden;
    white-space: nowrap;
    padding-right: 25px;
}

.ac_loading {
    background-image: url(/static/autocomplete/ajax-loader.gif);
    background-repeat: no-repeat;
    background-position: 3px 3px;
}

.ac_odd {
    background-color: #eee;
}

.ac_over {
    background-color: #3E4B5F;
    color: white;
}

.ac_results li.splash {
    background: url(/static/autocomplete/autocomplete-ajax.gif) no-repeat center center;
    height: 20px;
}

.ac_button {
    background-position: center center;
    background-image: url(/static/autocomplete/select-arrow.gif);
    background-repeat: no-repeat;
    width: 15px;
    height: 26px;
    padding: 1px;
    position: absolute;
    right: 0;
    top: 0;
    border-radius: 0 3px 3px 0;
    -moz-border-radius: 0 3px 3px 0;
    -webkit-border-radius: 0 3px 3px 0;
    cursor: pointer;
}
